@@ -208,6 +208,9 @@ cpl_table * irplib_slitpos_analysis(const cpl_image * imslit,
             cpl_msg_error(cpl_func, "cannot find the edges of the [%d]th line", 
                     i+1);
             cpl_image_delete(thin_im);
+            cpl_free(slit_y);
+            cpl_free(slit_x_l);
+            cpl_free(slit_x_r);
             return NULL;
         }
 
