@@ -65,13 +65,13 @@
 #ifdef CPL_MIN
 #define IRPLIB_MIN CPL_MIN
 #else
-#define IRPLIB_MIN(A,B) ((A) < (B) ? (A) : (B))
+#define IRPLIB_MIN(A,B) (((A) < (B)) ? (A) : (B))
 #endif
 
 #ifdef CPL_MAX
 #define IRPLIB_MAX CPL_MAX
 #else
-#define IRPLIB_MAX(A,B) ((A) > (B) ? (A) : (B))
+#define IRPLIB_MAX(A,B) (((A) > (B)) ? (A) : (B))
 #endif
 
 /*-----------------------------------------------------------------------------
@@ -102,9 +102,15 @@ static uint32_t irplib_roundup_power2(uint32_t v) CPL_ATTR_CONST;
 static
 cpl_error_code irplib_gaussian_maxpos(const cpl_image *,
                                       double,
+									  double,
+									  double,
                                       double *,
                                       double *,
                                       double *);
+
+static cpl_error_code
+irplib_closeset_aperture(const cpl_apertures * self,
+		const double x, const double y, int * ind);
 #endif
 
 /*-----------------------------------------------------------------------------
@@ -219,7 +225,7 @@ cpl_error_code irplib_strehl_compute(const cpl_image *   im,
     *psf_flux = 1.0; /* The psf flux, cpl_image_get_flux(psf), is always 1 */
 
 #ifndef IRPLIB_NO_FIT_GAUSSIAN
-    code = irplib_gaussian_maxpos(im, IRPLIB_STREHL_DETECT_LEVEL,
+    code = irplib_gaussian_maxpos(im, IRPLIB_STREHL_DETECT_LEVEL, xpos, ypos,
                                   &xposfit, &yposfit, &peak);
     if (code) {
         cpl_errorstate_set(prestate);
@@ -707,7 +713,9 @@ static double PSF_G(double  f,
 /*----------------------------------------------------------------------------*/
 static double PSF_sinc_norm(double x)
 {
-    return sin(x * CPL_MATH_PI) / (x * CPL_MATH_PI);
+    /* This static function should not be called with zero, but handle it
+       anyway. */
+    return x != 0.0 ? sin(x * CPL_MATH_PI) / (x * CPL_MATH_PI) : 1.0;
 }
 
 /*----------------------------------------------------------------------------*
@@ -893,6 +901,33 @@ static uint32_t irplib_roundup_power2(uint32_t v)
 }
 
 
+static cpl_error_code
+irplib_closeset_aperture(const cpl_apertures * self,
+		const double x, const double y, int * ind){
+
+	double dist = INFINITY;
+
+    const int nsize = cpl_apertures_get_size(self);
+
+    cpl_ensure_code(nsize > 0,    cpl_error_get_code());
+	cpl_ensure_code(ind,          CPL_ERROR_NULL_INPUT);
+
+	*ind = -1;
+
+	for(int i = 1; i <= nsize; ++i){
+		const double this_x = cpl_apertures_get_centroid_x(self, i);
+		const double this_y = cpl_apertures_get_centroid_y(self, i);
+		const double this_d_sq = pow(this_x - x, 2.0) + pow(this_y - y, 2.0);
+
+		if(this_d_sq < dist){
+			dist = this_d_sq;
+			*ind = i;
+		}
+
+	}
+	return CPL_ERROR_NONE;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Find the peak flux, peak sum and position of a Gaussian
@@ -907,7 +942,9 @@ static uint32_t irplib_roundup_power2(uint32_t v)
 /*----------------------------------------------------------------------------*/
 static
 cpl_error_code irplib_gaussian_maxpos(const cpl_image * self,
-                                      double sigma,
+                                      double    sigma,
+									  double    x_initial,
+									  double    y_initial,
                                       double  * pxpos,
                                       double  * pypos,
                                       double  * ppeak)
@@ -971,8 +1008,8 @@ cpl_error_code irplib_gaussian_maxpos(const cpl_image * self,
 
     aperts = cpl_apertures_new_from_image(self, labels);
 
-    /* Find the aperture with the greatest flux */
-    code = irplib_apertures_find_max_flux(aperts, &ifluxapert, 1);
+    /* Find the aperture closest to the provided coordinates */
+    code = irplib_closeset_aperture(aperts, x_initial, y_initial, &ifluxapert);
 
     if (code) {
         cpl_apertures_delete(aperts);
