@@ -305,31 +305,6 @@ static PilCdb *cfgDB = NULL;
 }
 
 
-/*
- * @brief
- *   Retrieve the path to the pipeline reduction block repository from the
- *   runtime environment.
- *
- * @return The function returns a string containing the pathname if no error
- *   occurred, otherwise it returns @c NULL.
- *
- * The function retrieves the path of a dataflow system specific 
- * directory from the environment. This directory specifies the location
- * of the reduction block repository. The path is copied to the, internally
- * created output string.
- *
- * The memory allocated for the returned string can be deallocated using
- * @b pil_free().
- */
-
- static char *pilDfsGetRbDir(void)
-{
-
-  return getpath(DFS_RBDIR);
-
-}
-
-
 /*
  * @brief
  *   Retrieve the path to the online archive system link directory from the
@@ -444,42 +419,6 @@ static PilCdb *cfgDB = NULL;
 
 }
 
-  
-/*
- * @brief
- *   Retrieve the pipeline display configuration from the runtime environment.
- * 
- * @return The function returns a string containing the display name if no
- *   error occurred, otherwise it returns @c NULL.
- *
- * The function checks if a pipeline specific display was specified
- * by searching the environment for the variable DFS_PIPE_DISPLAY.
- * If this variable is undefined DISPLAY is used instead. The display name
- * is copied to the, internally created, output string. 
- * If both variables do not exist the function returns @c NULL to indicate
- * that no graphics resources are available.
- * 
- * The memory allocated for the returned string can be deallocated using
- * @b pil_free().
- */
-
- static char *pilDfsGetDisplay(void)
-{
-
-  register char *value = getenv(PIL_DISPLAY);
-  
-
-  /*
-   * If no pipeline specific display is set use DISPLAY
-   */
-
-  if (!value)
-    if (!(value = getenv("DISPLAY")))
-      return NULL;
-
-  return pil_strdup(value);
-
-}
 
 
 /*
