@@ -1,8 +1,20 @@
 /*
- * hdrl_prototyping.c
+ * This file is part of the HDRL
+ * Copyright (C) 2014 European Southern Observatory
  *
- *  Created on: Aug 29, 2013
- *      Author: agabasch
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 
@@ -97,6 +109,7 @@ cpl_image * hdrl_get_spatial_freq(cpl_image *ima, double gausfilt, int mirrorx,
     filter_image = hdrl_gen_lowpass(xsize, ysize, sigma_x, sigma_y);
     if(filter_image == NULL){
         cpl_msg_error(cpl_func,"Filter image is NULL");
+        cpl_image_delete(eflat);
         return NULL;
     }
 
@@ -164,7 +177,6 @@ static cpl_image * hdrl_gen_lowpass(int xs, int ys, double sigma_x,
 	int hlx= 0.0;
 	int hly = 0.0;
 	double x= 0.0;
-	double y= 0.0;
 	double gaussval= 0.0;
 	float *data;
 
@@ -203,7 +215,7 @@ static cpl_image * hdrl_gen_lowpass(int xs, int ys, double sigma_x,
 	}
 
 	for (j=1; j<=hly ; j++) {
-		y = (double)j / sigma_y;
+		double y = (double)j / sigma_y;
 		/* first column */
 		data[j*xs] = (double)exp(-0.5*y*y);
 		data[(ys-j)*xs] = (double)exp(-0.5*y*y);
@@ -418,7 +430,7 @@ cpl_error_code hdrl_mime_compute_polynomial_bkg(
             cpl_matrix_delete(masked_image);
             cpl_matrix_delete(masked_tensors);
             cpl_image_delete(image);
-            cpl_error_set(cpl_func, CPL_ERROR_DATA_NOT_FOUND);
+            return cpl_error_set(cpl_func, CPL_ERROR_DATA_NOT_FOUND);
         }
 
         image_double = cpl_image_cast(image, CPL_TYPE_DOUBLE);
