@@ -919,14 +919,13 @@ void hdrl_cplvector_delete_to_cache(hdrl_vector_cache * cache, cpl_vector * v)
     if (!v) {
         return;
     }
-    if (!cache) {
-        return cpl_vector_delete(v);
-    }
-    cpl_size sz = cpl_vector_get_size(v);
-    if (sz < cache->nbuckets) {
-        if (cache->buckets[sz].available < cache->buckets[sz].nspace) {
-            cache->buckets[sz].ptrs[(cache->buckets[sz].available)++] = v;
-            return;
+    if (cache) {
+        const cpl_size sz = cpl_vector_get_size(v);
+        if (sz < cache->nbuckets) {
+            if (cache->buckets[sz].available < cache->buckets[sz].nspace) {
+                cache->buckets[sz].ptrs[(cache->buckets[sz].available)++] = v;
+                return;
+            }
         }
     }
     cpl_vector_delete(v);
@@ -1562,6 +1561,21 @@ cpl_mask * hcpl_image_set_bpm(cpl_image * self, cpl_mask * bpm)
 #endif
 }
 
+double hcpl_vector_get_mad_window(cpl_vector * vec,
+                                  cpl_size llx,
+                                  cpl_size urx,
+                                  double * sigma)
+{
+    /* TODO use cpl directly when PIPE-4792 is fixed */
+    struct _cpl_image_ img;
+    img.pixels = cpl_vector_get_data(vec);
+    img.nx = cpl_vector_get_size(vec);
+    img.ny = 1;
+    img.bpm = NULL;
+    img.type = CPL_TYPE_DOUBLE;
+    return cpl_image_get_mad_window(&img, llx, 1, urx, 1, sigma);
+}
+
 double hcpl_gaussian_eval_2d(const cpl_array * self, double x, double y)
 {
 #if CPL_VERSION_CODE < CPL_VERSION(6, 4, 0)
