@@ -234,7 +234,7 @@ static cpl_error_code hdrl_imagelist_to_vector_test(void)
         /* add a good image so potential uninitialized memory is included */
         cpl_imagelist_set(list, cpl_image_new(5, 6, CPL_TYPE_DOUBLE), 3);
         cpl_vector * o[5];
-        hdrl_imagelist_to_vector_row(list, 6, o);
+        hdrl_imagelist_to_vector_row(list, 6, o, NULL);
         cpl_test_error(CPL_ERROR_NONE);
         cpl_test_eq(cpl_vector_get_size(o[4]), 3);
         cpl_test_eq(cpl_vector_get_sum(o[4]), 200);
@@ -256,7 +256,7 @@ static cpl_error_code hdrl_imagelist_to_vector_test(void)
         /* add a good image so potential uninitialized memory is included */
         cpl_imagelist_set(list, cpl_image_new(5, 6, CPL_TYPE_INT), 3);
         cpl_vector * o[5];
-        hdrl_imagelist_to_vector_row(list, 6, o);
+        hdrl_imagelist_to_vector_row(list, 6, o, NULL);
         cpl_test_error(CPL_ERROR_NONE);
         cpl_test_eq(cpl_vector_get_size(o[4]), 3);
         cpl_test_eq(cpl_vector_get_sum(o[4]), 200);
@@ -512,51 +512,51 @@ void hdrl_string_test(void)
     cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
 
     s = hdrl_join_string(NULL, 1, "test");
-    cpl_test(strcmp(s, "test") == 0);
+    cpl_test_zero(strcmp(s, "test"));
     cpl_free(s);
 
     s = hdrl_join_string(NULL, 2, "test", "bla");
-    cpl_test(strcmp(s, "testbla") == 0);
+    cpl_test_zero(strcmp(s, "testbla"));
     cpl_free(s);
 
     s = hdrl_join_string(".", 1, "test");
-    cpl_test(strcmp(s, "test") == 0);
+    cpl_test_zero(strcmp(s, "test"));
     cpl_free(s);
 
     s = hdrl_join_string(".", 2, "test", "bla");
-    cpl_test(strcmp(s, "test.bla") == 0);
+    cpl_test_zero(strcmp(s, "test.bla"));
     cpl_free(s);
 
     s = hdrl_join_string("--", 3, "test", "bla", "val");
-    cpl_test(strcmp(s, "test--bla--val") == 0);
+    cpl_test_zero(strcmp(s, "test--bla--val"));
     cpl_free(s);
 
     s = hdrl_join_string("--", 3, "", "bla", "val");
-    cpl_test(strcmp(s, "bla--val") == 0);
+    cpl_test_zero(strcmp(s, "bla--val"));
     cpl_free(s);
 
     s = hdrl_join_string("--", 3, "test", "", "val");
-    cpl_test(strcmp(s, "test--val") == 0);
+    cpl_test_zero(strcmp(s, "test--val"));
     cpl_free(s);
 
     s = hdrl_join_string(",", 3, "test", NULL, "val");
-    cpl_test(strcmp(s, "test,val") == 0);
+    cpl_test_zero(strcmp(s, "test,val"));
     cpl_free(s);
 
     s = hdrl_join_string("--", 4, "", NULL, "val", "test");
-    cpl_test(strcmp(s, "val--test") == 0);
+    cpl_test_zero(strcmp(s, "val--test"));
     cpl_free(s);
 
     s = hdrl_join_string("--", 3, NULL, "bla", "val");
-    cpl_test(strcmp(s, "bla--val") == 0);
+    cpl_test_zero(strcmp(s, "bla--val"));
     cpl_free(s);
 
     s = hdrl_join_string("--", 3, "test", "bla", "");
-    cpl_test(strcmp(s, "test--bla") == 0);
+    cpl_test_zero(strcmp(s, "test--bla"));
     cpl_free(s);
 
     s = hdrl_join_string("--", 3, "test", "bla", NULL);
-    cpl_test(strcmp(s, "test--bla") == 0);
+    cpl_test_zero(strcmp(s, "test--bla"));
     cpl_free(s);
 }
 
@@ -566,15 +566,17 @@ void hdrl_pfilter_test(void)
     size_t nx = 100;
     size_t any[] = {13, 22, 47, 100, 103, 104, 202, 203, 204,
                     542, 1034, 1239};
-    size_t am[] = {3, 5, 7, 13};
-    for (size_t j = 0; j < ARRAY_LEN(am); j++) {
-        cpl_mask * m = cpl_mask_new(am[j], am[j]);
-        cpl_matrix * k = cpl_matrix_new(am[j], am[j]);
+    size_t amx[] = {1, 1, 3, 3, 5, 7, 13};
+    size_t amy[] = {1, 3, 1, 3, 5, 7, 13};
+    for (size_t j = 0; j < ARRAY_LEN(amx); j++) {
+        cpl_mask * m = cpl_mask_new(amx[j], amy[j]);
+        cpl_matrix * k = cpl_matrix_new(amx[j], amy[j]);
         cpl_mask_not(m);
         cpl_matrix_add_scalar(k, 1);
         for (size_t i = 0; i < ARRAY_LEN(any); i++) {
             size_t ny = any[i];
-            cpl_msg_info(cpl_func, "Testing ny %zu mask %zu", ny, am[j]);
+            cpl_msg_info(cpl_func, "Testing ny %zu mask %zu,%zu", ny,
+                         amx[j], amy[j]);
             cpl_image * img = cpl_image_new(nx, ny, CPL_TYPE_DOUBLE);
             double * d = cpl_image_get_data_double(img);
             for (size_t kindex = 0; kindex < nx * ny; kindex++) {
@@ -586,7 +588,8 @@ void hdrl_pfilter_test(void)
             cpl_image * orig_inp = cpl_image_duplicate(img);
             cpl_image * ro = cpl_image_duplicate(img);
             cpl_image * rp;
-            cpl_image_filter_mask(ro, img, m, CPL_FILTER_MEDIAN, CPL_BORDER_FILTER);
+            cpl_image_filter_mask(ro, img, m, CPL_FILTER_MEDIAN,
+                                  CPL_BORDER_FILTER);
             rp = hdrl_parallel_filter_image(img, NULL, m, CPL_FILTER_MEDIAN);
             cpl_test_image_abs(rp, ro, 0);
             /* input unchanged */
@@ -608,6 +611,96 @@ void hdrl_pfilter_test(void)
     }
 }
 
+void hdrl_pconvert_test(void)
+{
+    cpl_size nx = 223;
+    cpl_size ny = 223;
+    cpl_propertylist * plist = cpl_propertylist_new();
+    cpl_propertylist_update_int(plist, "NAXIS", 2);
+    cpl_propertylist_update_int(plist, "NAXIS1", nx);
+    cpl_propertylist_update_int(plist, "NAXIS2", ny);
+    cpl_propertylist_update_string(plist, "CTYPE1", "RA---ZPN");
+    cpl_propertylist_update_string(plist, "CTYPE2", "DEC--ZPN");
+    cpl_propertylist_update_double(plist, "CRVAL1", 149.947);
+    cpl_propertylist_update_double(plist, "CRVAL2", 2.205);
+    cpl_propertylist_update_double(plist, "CRPIX1", 5401.42);
+    cpl_propertylist_update_double(plist, "CRPIX2", 6834.89);
+    cpl_propertylist_update_double(plist, "CD1_1", 1.8072e-07);
+    cpl_propertylist_update_double(plist, "CD1_2", 9.4796e-05);
+    cpl_propertylist_update_double(plist, "CD2_1", -9.4820e-05);
+    cpl_propertylist_update_double(plist, "CD2_2", 2.0167e-07);
+    cpl_propertylist_update_double(plist, "PV2_1", 1.);
+    cpl_propertylist_update_double(plist, "PV2_2", 0.);
+    cpl_propertylist_update_double(plist, "PV2_3", 44.);
+    cpl_propertylist_update_double(plist, "PV2_4", 0.);
+    cpl_propertylist_update_double(plist, "PV2_5", -10300.);
+    cpl_propertylist_update_string(plist, "CUNIT1", "deg");
+    cpl_propertylist_update_string(plist, "CUNIT2", "deg");
+
+    cpl_wcs * wcs = cpl_wcs_new_from_propertylist(plist);
+    cpl_matrix * from = cpl_matrix_new(nx * ny, 2);
+    for (cpl_size y = 0; y < ny; y++) {
+        for (cpl_size x = 0; x < nx; x++) {
+            cpl_matrix_set(from, (y * nx) + x, 0, x);
+            cpl_matrix_set(from, (y * nx) + x, 1, y);
+        }
+    }
+    cpl_matrix * to, *to2, *toc, *toc2;
+    cpl_array * status, *statusc;
+    hdrl_wcs_convert(wcs, from, &to, &status, CPL_WCS_PHYS2WORLD);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_wcs_convert(wcs, from, &toc, &statusc, CPL_WCS_PHYS2WORLD);
+
+    cpl_test_matrix_abs(to, toc, 0.);
+    cpl_test_array_abs(status, statusc, 0.);
+    cpl_array_delete(status);
+    cpl_array_delete(statusc);
+
+    hdrl_wcs_convert(wcs, to, &to2, &status, CPL_WCS_WORLD2PHYS);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_wcs_convert(wcs, to, &toc2, &statusc, CPL_WCS_WORLD2PHYS);
+
+    cpl_test_matrix_abs(to2, toc2, 0.);
+    cpl_test_array_abs(status, statusc, 0.);
+
+    cpl_matrix_delete(to);
+    cpl_matrix_delete(toc);
+    cpl_matrix_delete(to2);
+    cpl_matrix_delete(toc2);
+    cpl_array_delete(status);
+    cpl_array_delete(statusc);
+
+    /* invalid phys as world input */
+    hdrl_wcs_convert(wcs, from, &to, &status, CPL_WCS_WORLD2PHYS);
+    cpl_test_error(CPL_ERROR_UNSPECIFIED);
+    cpl_wcs_convert(wcs, from, &toc, &statusc, CPL_WCS_WORLD2PHYS);
+    cpl_test_array_abs(status, statusc, 0.);
+
+    cpl_matrix_delete(to);
+    cpl_matrix_delete(toc);
+    cpl_array_delete(status);
+    cpl_array_delete(statusc);
+
+    /* check error */
+    hdrl_wcs_convert(NULL, NULL, NULL, NULL, CPL_WCS_PHYS2WORLD);
+    cpl_test_error(CPL_ERROR_NULL_INPUT);
+
+    hdrl_wcs_convert(wcs, from, &to, &status, (cpl_wcs_trans_mode)1421);
+    cpl_test_error(CPL_ERROR_UNSUPPORTED_MODE);
+
+    cpl_propertylist_erase(plist, "NAXIS2");
+    cpl_wcs_delete(wcs);
+    wcs = cpl_wcs_new_from_propertylist(plist);
+    hdrl_wcs_convert(wcs, from, &to, &status, CPL_WCS_PHYS2WORLD);
+    cpl_test_error(CPL_ERROR_UNSPECIFIED);
+    cpl_matrix_delete(to);
+    cpl_array_delete(status);
+
+    cpl_propertylist_delete(plist);
+    cpl_wcs_delete(wcs);
+    cpl_matrix_delete(from);
+}
+
 /*----------------------------------------------------------------------------*/
 /**
  @brief   Unit tests of utility module
@@ -624,6 +717,7 @@ int main(void)
     hdrl_path_test();
     hdrl_string_test();
     hdrl_pfilter_test();
+    hdrl_pconvert_test();
 
     return cpl_test_end(0);
 }
