@@ -63,6 +63,7 @@
 #include "vmstdfluxtable.h"
 #include "vmqcutils.h"
 #include "vmcpl.h"
+#include "vimos_dfs.h"
 
 
 static cxint vmifustandard(PilSetOfFrames *);
@@ -838,6 +839,13 @@ vmifustandard(PilSetOfFrames *sof)
 
   /*
    * Now go for the bias subtraction
+   * 
+   * NOTE: This includes the overscan subtraction if biasMethod = BIAS_ZMASTER,
+   * which is the default. Since the overscan region in the master bias has 
+   * been created with growOverscans, there is some extra noise added here,
+   * but this can probably be ignored. A better approach is to use the method
+   * used in the MOS pipeline: subtract overscan for each individual frame,
+   * then subtract bias.
    */
 
   cpl_msg_info(task, "Bias removal...");
@@ -1239,7 +1247,11 @@ vmifustandard(PilSetOfFrames *sof)
   deleteSetOfDescriptors(&(sciFov->descs), "ESO PRO ZERO*");
   deleteSetOfDescriptors(&(sciFov->descs), "CD1_*");  
   deleteSetOfDescriptors(&(sciFov->descs), "CD2_*");  
-  deleteSetOfDescriptors(&(sciFov->descs), "HISTORY*");  
+  deleteSetOfDescriptors(&(sciFov->descs), "HISTORY*");
+  //These keywords shouldn't be present in the input raw, but we nevertheless
+  //remove them in case they are present (See PIPE-5422)
+  deleteSetOfDescriptors(&(sciFov->descs), "PCOUNT");  
+  deleteSetOfDescriptors(&(sciFov->descs), "GCOUNT");  
 
   data = (float *)cpl_image_get_data(extracted);
   sciExtracted = newImage(xlen, ylen, data);
@@ -1258,6 +1270,10 @@ vmifustandard(PilSetOfFrames *sof)
   deleteSetOfDescriptors(&(sciExtracted->descs), "CD1_*");  
   deleteSetOfDescriptors(&(sciExtracted->descs), "CD2_*");  
   deleteSetOfDescriptors(&(sciExtracted->descs), "HISTORY*");  
+  //These keywords shouldn't be present in the input raw, but we nevertheless
+  //remove them in case they are present (See PIPE-5422)
+  deleteSetOfDescriptors(&(sciExtracted->descs), "PCOUNT");  
+  deleteSetOfDescriptors(&(sciExtracted->descs), "GCOUNT");  
 
   skydata = (float *)cpl_image_get_data(sky);
   skyExtracted = newImage(xlen, 1, skydata);
@@ -1274,6 +1290,10 @@ vmifustandard(PilSetOfFrames *sof)
   deleteSetOfDescriptors(&(skyExtracted->descs), "CD1_*");
   deleteSetOfDescriptors(&(skyExtracted->descs), "CD2_*");
   deleteSetOfDescriptors(&(skyExtracted->descs), "HISTORY*");
+  //These keywords shouldn't be present in the input raw, but we nevertheless
+  //remove them in case they are present (See PIPE-5422)
+  deleteSetOfDescriptors(&(skyExtracted->descs), "PCOUNT");  
+  deleteSetOfDescriptors(&(skyExtracted->descs), "GCOUNT");  
 
   stardata = (float *)cpl_image_get_data(star);
   starExtracted = newImage(xlen, 1, stardata);
@@ -1290,6 +1310,10 @@ vmifustandard(PilSetOfFrames *sof)
   deleteSetOfDescriptors(&(starExtracted->descs), "CD1_*");
   deleteSetOfDescriptors(&(starExtracted->descs), "CD2_*");
   deleteSetOfDescriptors(&(starExtracted->descs), "HISTORY*");
+  //These keywords shouldn't be present in the input raw, but we nevertheless
+  //remove them in case they are present (See PIPE-5422)
+  deleteSetOfDescriptors(&(starExtracted->descs), "PCOUNT");  
+  deleteSetOfDescriptors(&(starExtracted->descs), "GCOUNT");  
 
 
   writeIntDescriptor(&(sciExtracted->descs),
@@ -1841,7 +1865,7 @@ vmifustandard(PilSetOfFrames *sof)
   deleteImage(standardImage);
 
 
-  strlower(strcpy(sphotTableName, sphotTableTag));
+  vmstrlower(strcpy(sphotTableName, sphotTableTag));
   /* strcat(sphotTableName, ".TFITS"); */
   strcat(sphotTableName, ".fits");
 
@@ -2062,7 +2086,7 @@ vmifustandard(PilSetOfFrames *sof)
    * update the set of frames.
    */
 
-  strlower(strcpy(reducedName, reducedTag));
+  vmstrlower(strcpy(reducedName, reducedTag));
   strcat(reducedName, ".fits");
 
   if (createFitsImage(reducedName, sciExtracted, reducedTag)) {
@@ -2096,7 +2120,7 @@ vmifustandard(PilSetOfFrames *sof)
   deleteImage(sciExtracted);
   cpl_image_delete(extracted);
 
-  strlower(strcpy(fovName, fovTag));
+  vmstrlower(strcpy(fovName, fovTag));
   strcat(fovName, ".fits");
 
   if (createFitsImage(fovName, sciFov, fovTag)) {
@@ -2127,7 +2151,7 @@ vmifustandard(PilSetOfFrames *sof)
   deleteImage(sciFov);
   cpl_image_delete(fov);
 
-  strlower(strcpy(skyName, skyTag));
+  vmstrlower(strcpy(skyName, skyTag));
   strcat(skyName, ".fits");
 
   if (createFitsImage(skyName, skyExtracted, skyTag)) {
@@ -2155,7 +2179,7 @@ vmifustandard(PilSetOfFrames *sof)
   deleteImage(skyExtracted);
   cpl_image_delete(sky);
 
-  strlower(strcpy(starName, starTag));
+  vmstrlower(strcpy(starName, starTag));
   strcat(starName, ".fits");
 
   if (createFitsImage(starName, starExtracted, starTag)) {
