@@ -66,6 +66,7 @@ vimos::flat_normaliser::~flat_normaliser()
  * @param spa_fit_polyorder   Order for polynomial fit along spatial axis
  * @param disp_fit_knots      Number of knots in the spline fitting along dispersion axis
  * @param fit_threshold       values below this will be igonred in the fits 
+ * @param normalise_spa_local if TRUE spatial normalisation is done row-by-row
  *  
  * @return The smoothed flat field exposure used for normalisation
  *
@@ -94,7 +95,7 @@ int vimos::flat_normaliser::mos_normalise
  double dispersion,
  int spa_smooth_radius, int disp_smooth_radius, 
  int spa_fit_polyorder, int disp_fit_nknots,
- double fit_threshold)
+ double fit_threshold, bool normalise_spa_local)
 {
     const char     *func = "mos_mosflat_normalise";
 
@@ -248,11 +249,12 @@ int vimos::flat_normaliser::mos_normalise
         
         std::vector<float> slit_spa_norm_profile;
         std::vector<float> slit_disp_norm_profile;
-        mosca::image normslit = mosca::image_normalise(im_exslit_orig,
+        mosca::image normslit = mosca::image_normalise_spa_local(im_exslit_orig,
                                       slit_mask_im_mos,
                                       final_spa_smooth_radius, disp_smooth_radius,
                                       spa_fit_polyorder, disp_fit_nknots, 
-                                      fit_threshold, slit_spa_norm_profile,
+                                      fit_threshold, normalise_spa_local,
+                                      slit_spa_norm_profile,
                                       slit_disp_norm_profile);
 
         //Get a position around the middle of the slit with a valid wave calib
@@ -268,9 +270,16 @@ int vimos::flat_normaliser::mos_normalise
         if(pix_waveref_left>=0 && pix_waveref_rigth < flat.size_dispersion())
             prof_val_waveref = (slit_disp_norm_profile[pix_waveref_left] +
                                 slit_disp_norm_profile[pix_waveref_rigth]) / 2.;
-        for(size_t i_prof = 0; i_prof < slit_disp_norm_profile.size(); i_prof++)
-            slit_disp_norm_profile[i_prof] /= prof_val_waveref;
-        
+
+        if(prof_val_waveref == 0.0)
+        	cpl_msg_warning(cpl_func, "There is no flux at the reference wavelength for slit %i", slit_id[i]);
+
+        for(size_t i_prof = 0; i_prof < slit_disp_norm_profile.size(); i_prof++){
+            if(prof_val_waveref != 0.0)
+            	slit_disp_norm_profile[i_prof] /= prof_val_waveref;
+            else
+            	slit_disp_norm_profile[i_prof] = 0.0;
+        }
         m_wave_profiles.push_back(slit_disp_norm_profile);
         m_wave_profiles_norm.push_back(prof_val_waveref);
         
@@ -374,6 +383,9 @@ int vimos::flat_normaliser::mos_normalise
          */
         int disp_bottom, spa_bottom, disp_top, spa_top;
         slit_it->get_extent_pix(disp_bottom, spa_bottom, disp_top, spa_top);
+
+        spa_bottom = std::max(0, spa_bottom);
+        spa_top = std::min(ny - 1, spa_top);
         for (j = start_pixel; j < end_pixel; j++) 
         {
             for (int yint = spa_bottom; yint <= spa_top; yint++) 
@@ -462,6 +474,15 @@ const std::vector<std::vector<float> >& vimos::flat_normaliser::get_wave_profile
     return m_wave_profiles;
 }
 
+static
+void dump_vector(const std::vector<float>& v){
+    std::stringstream ss;
+    for(const auto& e : v){
+        ss<< e <<" ";
+    }
+    cpl_msg_info(cpl_func, "For FLAT SED normalization the following widths are used %s", ss.str().c_str());
+}
+
 std::vector<float> vimos::flat_normaliser::get_wave_profiles_norm
 (double mflat_exptime,
  const std::vector<float>& slit_widths,
@@ -472,9 +493,11 @@ std::vector<float> vimos::flat_normaliser::get_wave_profiles_norm
         throw std::invalid_argument("Vector sizes do not match");
     std::vector<float> wave_profiles_norm_scaled;
     
+    dump_vector(slit_widths);
+
     for(size_t i = 0; i < m_wave_profiles_norm.size(); ++i)
     {
-        float scale = mflat_exptime * slit_widths[i] * slit_lengths[i];
+        float scale = mflat_exptime * slit_widths[i];
         if(scale == 0 )
             scale = 1;  //In some cases the length has been detected as 0.
         wave_profiles_norm_scaled.push_back(m_wave_profiles_norm[i] / scale);
@@ -522,12 +545,13 @@ cpl_image * vimos::flat_normaliser::get_wave_profiles_im_mapped
                     (wave_cal, slit_end_pos, slit_begin_pos);
             for(cpl_size i_wave = 0; i_wave < nl; ++i_wave)
             {
+            	const auto& p = m_wave_profiles[i_slit];
                 double wave = firstLambda + i_wave * dispersion;
                 double pixel = wave_cal.get_pixel(slit_pos_good_wavecal, wave);
                 int i_pix = std::ceil(pixel+0.5);
                 if(i_pix>= 0 && i_pix <m_wave_profiles.front().size())
                     cpl_image_set(mapped_flat_sed, i_wave+1, i_slit+1,
-                                  m_wave_profiles[i_slit][i_pix]);
+                                  p[i_pix]);
             }
         }
     }
