@@ -6,10 +6,13 @@
 #include "vimos_science_impl.h"
 #include "vimos_dfs.h"
 #include "irplib_sdp_spectrum.h"
+#include <moses.h>
 #include "math.h"
 #include <algorithm>
 #include <vector>
 
+#define PI 3.14159265358979323846
+
 /*----------------- Private Structs Declarations -------------------------*/
 
 const double angstrom_to_nm = 0.1;
@@ -44,15 +47,19 @@ typedef std::pair<double, double> ra_dec;
 
 /*---------------------- Forward Declarations ------------------------------*/
 
-static cpl_error_code
-save_spectra(idp_save_info info, const cpl_size mplex,
+static
+double get_property_or_nan(const cpl_propertylist * l, const char * name);
+
+static
+cpl_error_code save_spectra(idp_save_info& info, const cpl_size mplex,
 		const std::vector<ra_dec>& ra_decs_for_extension);
 
-static bool
-get_ra_dec_for_extension(cpl_frameset * frames, const cpl_size multiplex_idx,
-		std::vector<ra_dec>& data_out);
+static
+bool get_ra_dec_for_extension(cpl_frameset * frames,
+                const cpl_size multiplex_idx, std::vector<ra_dec>& data_out);
 
-static double extract_aperture_for_extension(const cpl_frame * science,
+static
+double extract_aperture_for_extension(const cpl_frame * science,
 		const cpl_size index);
 
 static
@@ -63,7 +70,7 @@ static
 cpl_array * get_wlens_angstrom(const cpl_frame * frame, const cpl_size num_wav);
 
 static
-bool check_posag_from_science(const cpl_frameset * fset);
+bool check_posang_from_science(const cpl_frameset * fset);
 
 /*---------------------------- Public Functions ------------------------------*/
 
@@ -93,7 +100,7 @@ cpl_error_code vimos_spec_idp_save(cpl_parameterlist * parlist, cpl_frameset * f
 
     if(!generate) return CPL_ERROR_NONE;
 
-    if(!check_posag_from_science(frames))
+    if(!check_posang_from_science(frames))
         return CPL_ERROR_NONE;
 
 
@@ -311,20 +318,21 @@ const char * get_grism_name_tag(const cpl_propertylist * science_header){
 /*---------------------------- Private Functions ------------------------------*/
 
 static const char * wave_units = "angstrom";
-static const char * flx_units = "10**(-16) erg cm**(-2) sec**(-1) angstrom**(-1)";
-static const char * flx_units_uncalib = "adu sec**(-1)";
+static const char * flx_units = "10**(-16)erg.cm**(-2).s**(-1).angstrom**(-1)";
+static const char * flx_units_uncalib = "adu.s**(-1)";
 
+static const char* type_qual = "spec:Data.FluxAxis.Accuracy.QualityStatus";
 static const char* type_wave = "spec:Data.SpectralAxis.Value";
 static const char* type_flx = "spec:Data.FluxAxis.Value";
 static const char* type_flx_err = "spec:Data.FluxAxis.Accuracy.StatError";
 static const char* type_flx_uncalib = "eso:Data.FluxAxis.Value";
 static const char* type_flx_err_uncalib = "eso:Data.FluxAxis.Accuracy.StatError";
 
+static const char * ucd_qual = "meta.code.qual;meta.main";
 static const char * ucd_wave = "em.wl;obs.atmos";
-static const char * ucd_flux = "phot.ux.density;em.wl;src.net;meta.main";
-static const char * ucd_flux_err = "stat.error;phot.ux.density;meta.main";
-
-static const char * ucd_flux_uncalib = "phot.ux.density;em.wl;src.net;stat.uncalib";
+static const char * ucd_flux = "phot.flux.density;em.wl;src.net;meta.main";
+static const char * ucd_flux_err = "stat.error;phot.flux.density;meta.main";
+static const char * ucd_flux_uncalib = "phot.flux.density;em.wl;src.net;stat.uncalib";
 static const char * ucd_flux_err_uncalib = "stat.error;phot.flux.density;stat.uncalib";
 
 static const char * wave_clmn_name = "WAVE";
@@ -336,7 +344,7 @@ extract_flux(const cpl_image * img, const cpl_size idx){
 
 	const cpl_size sz_x = cpl_image_get_size_x(img);
 	cpl_array * to_ret = cpl_array_new(sz_x, CPL_TYPE_DOUBLE);
-	for(cpl_size x = 0; x < cpl_image_get_size_x(img); ++x){
+	for(cpl_size x = 0; x < sz_x; ++x){
 		int rej = 0;
 		const double val = cpl_image_get(img, x + 1, idx + 1, &rej);
 		if(!rej)
@@ -348,8 +356,23 @@ extract_flux(const cpl_image * img, const cpl_size idx){
 	return to_ret;
 }
 
+static cpl_array *
+make_qual(const cpl_array * flx_data) {
+    const cpl_size flux_sz = cpl_array_get_size(flx_data);
+    cpl_array * qual_data = cpl_array_new(flux_sz, CPL_TYPE_INT);
+    for (cpl_size x = 0; x < flux_sz; ++x) {
+        int reject = 0;
+        const double val = cpl_array_get(flx_data, x, &reject);
+        if (reject)  // 1 datum is invalid, -1 there was an error
+            cpl_array_set_invalid(qual_data, x);
+        else
+            cpl_array_set(qual_data, x, val == -1.0 ? 1 : 0);
+    }
+    return qual_data;
+}
+
 static
-irplib_sdp_spectrum * extract_1D_spectrum(idp_save_info info, const cpl_size idx){
+irplib_sdp_spectrum * extract_1D_spectrum(idp_save_info& info, const cpl_size idx){
 
 	irplib_sdp_spectrum * spectrum = irplib_sdp_spectrum_new();
 	const cpl_size sz = cpl_image_get_size_x(info.science_img);
@@ -368,6 +391,8 @@ irplib_sdp_spectrum * extract_1D_spectrum(idp_save_info info, const cpl_size idx
 	cpl_array * flx_red_data = extract_flux(info.science_img, idx);
 	cpl_array * flx_red_data_err = extract_flux(info.science_img_err, idx);
 
+    cpl_array * qual_data = make_qual(flx_data);
+
 	fail = irplib_sdp_spectrum_add_column(spectrum, wave_clmn_name, CPL_TYPE_DOUBLE, wave_units,
 			NULL, type_wave, ucd_wave, info.wlens_angstrom);
 
@@ -398,7 +423,7 @@ irplib_sdp_spectrum * extract_1D_spectrum(idp_save_info info, const cpl_size idx
 		return NULL;
 	}
 
-	fail = irplib_sdp_spectrum_add_column(spectrum, "SKYBACK", CPL_TYPE_DOUBLE, flx_units_uncalib,
+	fail = irplib_sdp_spectrum_add_column(spectrum, "BGFLUX", CPL_TYPE_DOUBLE, flx_units_uncalib,
 			NULL, type_flx_uncalib, ucd_flux_uncalib, bkg_data);
 
 	if(fail){
@@ -422,6 +447,14 @@ irplib_sdp_spectrum * extract_1D_spectrum(idp_save_info info, const cpl_size idx
 		return NULL;
 	}
 
+	fail = irplib_sdp_spectrum_add_column(spectrum, "QUAL", CPL_TYPE_INT, "",
+			NULL, type_qual, ucd_qual, qual_data);
+
+	if(fail){
+		irplib_sdp_spectrum_delete(spectrum);
+		return NULL;
+	}
+
 	cpl_array_delete(flx_data);
 	cpl_array_delete(flx_err_data);
 
@@ -430,6 +463,8 @@ irplib_sdp_spectrum * extract_1D_spectrum(idp_save_info info, const cpl_size idx
 
 	cpl_array_delete(bkg_data);
 
+	cpl_array_delete(qual_data);
+
 	return spectrum;
 }
 
@@ -800,7 +835,7 @@ static double get_stacked_effron(const double detron,
     return detron / sqrt(nstacked);
 }
 
-static bool add_ron_from_bias(irplib_sdp_spectrum * dest, idp_save_info info,
+static bool add_ron_from_bias(irplib_sdp_spectrum * dest, idp_save_info& info,
         const cpl_propertylist * science_plist){
 
     const char * qc_ron_tag = "ESO QC RON";
@@ -827,6 +862,53 @@ static bool add_stacking_pars(irplib_sdp_spectrum * dest, const cpl_propertylist
 	return success;
 }
 
+/* makes use of crder, so must run after add_drs_err() */
+static
+bool add_radec_err(cpl_propertylist * dest, cpl_propertylist * source) {
+    // VLT positioning rms error [3 arcsec] projected equally along RA and DEC
+    // rms positioning error: 3 arcsec
+    const double err_vlt_ra = 3. / 3600. / sqrt(2.);
+    const double err_vlt_dec = 3. / 3600. / sqrt(2.);
+
+    const double posang = get_property_or_nan(source, "ESO ADA POSANG");
+    const double crder = get_property_or_nan(dest, idp_output_crder_tag);
+    if (std::isnan(posang) || std::isnan(crder)) return false;
+
+    const double cos_posang = cos(posang * PI / 180.0);
+    const double sin_posang = sin(posang * PI / 180.0);
+
+    // the error due to the wavelength calibration projected along the RA and
+    // DEC axes
+    const double err_disp_ra = std::abs(crder / 3600. * sin_posang);
+    const double err_disp_dec = std::abs(crder / 3600. * cos_posang);
+
+    cpl_table * slits = mos_load_slits_vimos(source, 0);
+    if (!slits) return false;
+    const double med_slit_width = cpl_table_get_column_median(slits, "ywidth");
+    cpl_table_delete(slits);
+
+    // the possible offset of a target from the center of the slit along the
+    // dispersion axis (0.3 x slit width [arcsec]), projected along the RA and
+    // DEC axes
+    const double err_slit_ra = abs((med_slit_width / 3.) / 3600. * sin_posang);
+    const double err_slit_dec = abs((med_slit_width / 3.) / 3600. * cos_posang);
+
+    const double ra_err = sqrt(pow(err_vlt_ra, 2.) + pow(err_disp_ra, 2.) +
+            pow(err_slit_ra, 2.));
+    const double dec_err = sqrt(pow(err_vlt_dec, 2.) + pow(err_disp_dec, 2.) +
+            pow(err_slit_dec, 2.));
+
+    bool success = !cpl_propertylist_update_double(dest, "RA_ERR", ra_err);
+    success = success && !cpl_propertylist_set_comment(dest, "RA_ERR",
+            "Error on spectroscopic target position [deg]");
+    success = success && !cpl_propertylist_update_double(dest, "DEC_ERR",
+            dec_err);
+    success = success && !cpl_propertylist_set_comment(dest, "DEC_ERR",
+            "Error on spectroscopic target position [deg]");
+
+    return success;
+}
+
 static bool add_filter(cpl_propertylist * dest, const cpl_propertylist * source){
 
 	const char * filt_name_tag = get_filt_name_tag(source);
@@ -849,7 +931,7 @@ static bool add_filter(cpl_propertylist * dest, const cpl_propertylist * source)
 
 static bool
 add_from_master_screen_flat(irplib_sdp_spectrum * dest,
-		idp_save_info info,
+		idp_save_info& info,
 		const cpl_size extension_idx){
 
 	const char * tag = "MOS_MASTER_SCREEN_FLAT";
@@ -892,15 +974,15 @@ static bool add_drs_err(cpl_propertylist * spec_plist,
 
 	if(std::isnan(crder)) return false;
 
-	bool success = !cpl_propertylist_update_double(spec_plist, idp_output_csyer_tag, 5.0);
-	success = success && !cpl_propertylist_set_comment(spec_plist, idp_output_csyer_tag, "Systematic error in position");
+	bool success = !cpl_propertylist_update_double(spec_plist, idp_output_csyer_tag, 3.0);
+	success = success && !cpl_propertylist_set_comment(spec_plist, idp_output_csyer_tag, "Systematic error in position [arcsec]");
 	success = success && !cpl_propertylist_update_double(spec_plist, idp_output_crder_tag, crder);
-	success = success && !cpl_propertylist_set_comment(spec_plist, idp_output_crder_tag, "Random error in position");
+	success = success && !cpl_propertylist_set_comment(spec_plist, idp_output_crder_tag, "Random error in position [arcsec]");
 
 	return success;
 }
 
-static cpl_error_code append_metadata(irplib_sdp_spectrum * dest, idp_save_info info,
+static cpl_error_code append_metadata(irplib_sdp_spectrum * dest, idp_save_info& info,
 		const cpl_size multiplex_idx, cpl_propertylist * spec_plist, const double ra,
 		const double dec){
 
@@ -932,6 +1014,7 @@ static cpl_error_code append_metadata(irplib_sdp_spectrum * dest, idp_save_info
 	const double lam_rms = irplib_sdp_spectrum_get_lamrms(dest);
 
 	success = add_drs_err(spec_plist, science_plist, lam_rms);
+	success = success && add_radec_err(spec_plist, science_plist);
 
 	cpl_propertylist_delete(science_plist);
 
@@ -940,7 +1023,7 @@ static cpl_error_code append_metadata(irplib_sdp_spectrum * dest, idp_save_info
 }
 
 static cpl_error_code
-save_spectrum(idp_save_info info, const cpl_size spec_idx, const cpl_size multiplex_ex,
+save_spectrum(idp_save_info& info, const cpl_size spec_idx, const cpl_size multiplex_ex,
 		const double ra, const double dec){
 
 	if(std::isnan(ra) || std::isnan(dec)){
@@ -988,7 +1071,7 @@ save_spectrum(idp_save_info info, const cpl_size spec_idx, const cpl_size multip
 }
 
 static cpl_error_code
-save_spectra(idp_save_info info, const cpl_size mplex, const std::vector<ra_dec>& ra_decs_for_extension){
+save_spectra(idp_save_info& info, const cpl_size mplex, const std::vector<ra_dec>& ra_decs_for_extension){
 
 	const cpl_size sz_y = cpl_image_get_size_y(info.science_img);
 	for(cpl_size y = 0; y < sz_y; ++y){
@@ -1162,13 +1245,13 @@ static double extract_aperture_for_extension(const cpl_frame * science, const cp
 	std::size_t target_idx = slits_dim_y.size() / 2;
 	std::vector<double>::iterator target = slits_dim_y.begin() + target_idx;
 	std::nth_element(slits_dim_y.begin(), target, slits_dim_y.end());
-	const double aperture = slits_dim_y[target_idx];
+	const double aperture = slits_dim_y[target_idx] / 3600.0;
 
 	return aperture;
 }
 
 static
-bool check_posag_from_science(const cpl_frameset * fset){
+bool check_posang_from_science(const cpl_frameset * fset){
 
     cpl_propertylist * science_header = dfs_load_header(fset, "MOS_SCIENCE", 0);
     if(science_header == NULL) return false;
