@@ -42,6 +42,7 @@
 #include "vmtable.h"
 #include "vmwindowtable.h"
 #include "vmobjecttable.h"
+#include "cpl.h"
 
 
 /*
@@ -53,7 +54,7 @@ VimosObjectObject *newObjectObject(void)
   VimosObjectObject *newObject;
   
   /* allocate memory for Object Object */
-  newObject = (VimosObjectObject *) pil_malloc(sizeof(VimosObjectObject));
+  newObject = (VimosObjectObject *) cpl_malloc(sizeof(VimosObjectObject));
   
   /* check if space was allocated */
   if (newObject == NULL) {
@@ -95,7 +96,7 @@ void deleteObjectObject(VimosObjectObject *obj)
     /* get address of next object */
     nextObj = tmpObj->next;
     /* free current object */
-    pil_free(tmpObj);
+    cpl_free(tmpObj);
     /* next one to process */
     tmpObj = nextObj;
   }
@@ -112,7 +113,7 @@ VimosObjectTable *newObjectTable()
   VimosObjectTable *newTab;
   
   /* allocate new VimosTable */
-  newTab = (VimosObjectTable *) pil_malloc(sizeof(VimosObjectTable));
+  newTab = (VimosObjectTable *) cpl_malloc(sizeof(VimosObjectTable));
 
   /* check if space was allocated */
   if (newTab == NULL) {
@@ -125,7 +126,7 @@ VimosObjectTable *newObjectTable()
   newTab->descs = newStringDescriptor("ESO PRO TABLE", VM_OBJ, "");
   if (newTab->descs == NULL) {
     /* cleanup */
-    pil_free(newTab);
+    cpl_free(newTab);
     cpl_msg_error("newObjectTable","The function newStringDescriptor has returned NULL");
     return(NULL);
   }
@@ -286,9 +287,9 @@ VimosBool readFitsObjectTable(VimosObjectTable *objTable, fitsfile *fptr)
   }
 
  /* allocate space for the column labels */
-  ttype = (char **) pil_malloc(nCols*sizeof(char *));
+  ttype = (char **) cpl_malloc(nCols*sizeof(char *));
   for (ii = 0; ii < nCols; ii++) {
-    ttype[ii] = (char *) pil_malloc(FLEN_VALUE*sizeof(char));
+    ttype[ii] = (char *) cpl_malloc(FLEN_VALUE*sizeof(char));
     /* check if space was allocated */
     if (ttype[ii] == NULL) {
       cpl_msg_error("readFitsObjectTable","Allocation Error");
@@ -408,7 +409,7 @@ VimosBool readFitsObjectTable(VimosObjectTable *objTable, fitsfile *fptr)
   }
    
   for (ii = 0; ii < 3; ii++){     /* free the memory for the column labels */
-    pil_free( ttype[ii] );
+    cpl_free( ttype[ii] );
   }
 
   return (VM_TRUE);
@@ -458,13 +459,13 @@ VimosBool writeFitsObjectTable(VimosObjectTable *objTable, fitsfile *fptr)
   }
  
   for (i = 0; i <= 8; i++){      /* allocate space for the column labels */
-    ttype[i] = (char *) pil_malloc(FLEN_VALUE);  /* max label length = 69 */
+    ttype[i] = (char *) cpl_malloc(FLEN_VALUE);  /* max label length = 69 */
     /* check if space was allocated */
     if (ttype[i] == NULL) {
       cpl_msg_error("writeFitsObjectTable","Allocation Error");
       return(VM_FALSE);
     }
-    tform[i] = (char *) pil_malloc(FLEN_VALUE);
+    tform[i] = (char *) cpl_malloc(FLEN_VALUE);
     /* check if space was allocated */
     if (tform[i] == NULL) {
       cpl_msg_error("writeFitsObjectTable","Allocation Error");
