@@ -89,19 +89,23 @@ static char *_get_base_name(const char *filename)
 const char * vimos_get_license(void)
 {
     const char  *   vimos_license = 
-        "This file is currently part of the VIMOS Instrument Pipeline\n"
-        "Copyright (C) 2002-2011 European Southern Observatory\n\n"
+        "This file is part of the VIMOS Instrument Pipeline\n"
+        "Copyright (C) 2019 European Southern Observatory\n"
+        "\n"
         "This program is free software; you can redistribute it and/or modify\n"
         "it under the terms of the GNU General Public License as published by\n"
         "the Free Software Foundation; either version 2 of the License, or\n"
-        "(at your option) any later version.\n\n"
+        "(at your option) any later version.\n"
+        "\n"
         "This program is distributed in the hope that it will be useful,\n"
         "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n"
-        "GNU General Public License for more details.\n\n"
+        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+        "GNU General Public License for more details.\n"
+        "\n"
         "You should have received a copy of the GNU General Public License\n"
-        "along with this program; if not, write to the Free Software Foundation,\n"
-        "Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA\n";
+        "along with this program; if not, write to the Free Software\n"
+        "Foundation, Inc., 59 Temple Place, Suite 330, Boston, \n"
+        "MA  02111-1307  USA";
     return vimos_license;
 }
 
